#!/usr/bin/env python

import __version__
__version__ = __version__.ver

from wxPython.wx import *
from App import *
from About import AboutBox
from Help import HelpBox
from tempfile import mktemp
import ftplib
import string
import os
import xmlrpclib
import httplib
from base64 import encodestring

# your favorite editor
# NOTE: - the %s is replaced by the filename of the tmpfile. 
#       - if you are using Win?? you need to have '\\' in any path names
#         instead of a single '\'
#         e.g. 'C:\\WINNT\\system32\\notepad.exe %s'
EDITOR_CMD = 'xjed %s -l ./zope.sl'
# change these to suit your system
HOST = 'localhost'
PORT = '8021'
USER = 'admin'
PASSWD = 'yourpassword'
# where to download files
# this is only for SHIFT-Right-Click downloads
# you might need something like  'C:\\window\\temp' for Win??
DOWNLOAD_PATH = './'
# change this to 0 if you don't want to see the open dialog
USE_OPEN_DIALOG = 1

# no need to change the rest of these
TMP_FILE_SUFFIX = 'ze'


ZopeEDIT_menu = """<?xml version="1.0"?>
<wxappdef title="ZopeEDIT" wide="600" high="400" statusbar="default">
  <menudef>
    <menu name="&amp;File">
       <menuitem name="&amp;Open" accel="Ctrl-O" id="1001" tip="Connect to Medusa" />
       <menuitem name="&amp;Close" id="1004" tip="Close connection" />
       <menuseparator />
       <menuitem name="E&amp;xit" accel="Ctrl-X" id="1006" />
    </menu>
    <menu name="&amp;Edit">
       <menuitem name="&amp;New" accel="Ctrl-N" id="2001" tip="Add new DTML Document" />
       <menuitem name="&amp;NewFolder" accel="Ctrl-F" id="2002" tip="Add new DTML Document" />
       <menuseparator />
       <menuitem name="Cu&amp;t" accel="Alt-X" id="2003" tip="Delete and save document" />
       <menuitem name="&amp;Copy" accel="Alt-C" id="2004" tip="Copy document" />
       <menuitem name="&amp;Paste" accel="Alt-V" id="2005" tip="Paste document" />
    </menu>
    <menu name="&amp;Help">
       <menuitem name="&amp;Help" accel="F1" id="3001" tip="Get help now!" />
       <menuseparator />
       <menuitem name="&amp;About" id="3002" tip="Who dunnit" />
    </menu>
  </menudef>
  <toolbar>
    <toolbaritem name="Open" bitmap="Open" id="1001" tip="Open connection" />
    <toolbaritem name="Close" bitmap="Close" id="1004" tip="Close connection" />
    <toolbarseparator />
    <toolbaritem name="New" bitmap="New" id="2001" tip="New DTML document" />
    <toolbaritem name="Cut" bitmap="Cut" id="2002" tip="Delete and save document" />
    <toolbaritem name="Copy" bitmap="Copy" id="2003" tip="Copy document" />
    <toolbaritem name="Paste" bitmap="Paste" id="2004" tip="Paste document" />
  </toolbar>
</wxappdef>
"""

# for XML-RPC with auth

# NOTES: We can't get enough info with RPC without adding some info funcs
# to OFS/SimpleItem. Perhaps we can add some globals a la RDF, but RDF is
# a parse-pig and I don't want to spend much time writing a parser class
# for it. DC may have a parser in python that could be used. They have
# added a bunch of proprietary tags for thier IDE which will no doubt change
# a lot. Pain in the butt! It would be easy to write some simple Product
# that allows getting at the object info. How does AUTH work with this???
class BasicAuthTransport( xmlrpclib.Transport ):
    def __init__( self, username=None, password=None ):
	self.username = username
	self.password = password
	
    # --------------------------------------------------------------------
    def request( self, host, handler, request_body ):
	h = httplib.HTTP( host )
	h.putrequest( "POST", handler )
	
	h.putheader("Host", host )
	
	h.putheader( "User-Agent", self.user_agent )
	h.putheader( "Content-Type", "text/xml" )
	h.putheader( "Content-Length", str(len(request_body)))
	
	if self.username is not None and self.password is not None:
	    h.putheader("AUTHORIZATION", "Basic %s" % string.replace(encodestring("%s:%s" % (self.username, self.password)), "\012", "" ))
	h.endheaders()
	
	if request_body:
	    h.send( request_body )
	    
	errcode, errmsg, headers = h.getreply()
	
	if errcode != 200:
	    raise xmlrpclib.ProtocolError( host + handler, errcode, errmsg, headers )
	
	return self.parse_response( h.getfile() )

# ========================================================================
class ZopeEDIT( App ):
    def __init__( self, resource ):
	self._ftp = ftplib.FTP()
	self._root = None
	# for the progress bar
	self._progress = None
	self._currentsize = 0
	self._rpc = None
	self._curpath = '/'
	self._curitem = -1
	
	App.__init__( self, resource )
	
	self._img_list = wxImageList( 16, 16 )
	self.img_folder  = self._img_list.Add(wxBitmap('Images/folder_green.bmp', wxBITMAP_TYPE_BMP))
	self.img_folder_open  = self._img_list.Add(wxBitmap('Images/folder-open.bmp', wxBITMAP_TYPE_BMP))
	self.img_text  = self._img_list.Add(wxBitmap('Images/text.bmp', wxBITMAP_TYPE_BMP))
    
	
	splitter = wxSplitterWindow( self, -1 )
	tID = wxNewId()
	self.tree = wxTreeCtrl( splitter, tID )
	self.tree.SetImageList( self._img_list )
	EVT_TREE_ITEM_EXPANDED( self.tree, tID, self.OnItemExpanded )
	EVT_TREE_ITEM_COLLAPSED( self.tree, tID, self.OnItemCollapsed )
	EVT_TREE_SEL_CHANGED( self.tree, tID, self.OnTreeSelChanged )
	
	listID = wxNewId()
	self.flist = wxListCtrl( splitter, listID, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxSUNKEN_BORDER|wxLC_SINGLE_SEL|wxLC_AUTOARRANGE )
	self.flist.SetImageList( self._img_list, wxIMAGE_LIST_SMALL )
	self.flist.InsertColumn( 0, "Id" )
	self.flist.InsertColumn( 1, "Title" )
	EVT_LIST_ITEM_SELECTED( self.flist, listID, self.OnListItemSelected )
	EVT_LEFT_DCLICK( self.flist, self.OnListDClick )
	EVT_RIGHT_DOWN( self.flist, self.OnListRightDown )
	# wxMSW
	EVT_COMMAND_RIGHT_CLICK( self.flist, listID, self.OnListRightClick )
	# wxGTK
	EVT_RIGHT_UP( self.flist, self.OnListRightClick )
#	EVT_LISTBOX_DCLICK( self.flist, listID, self.OnListDClick )
#	EVT_RIGHT_UP( self.flist, self.OnListRightClick )
	
	splitter.SplitVertically( self.tree, self.flist )
	splitter.SetSashPosition( 180, true )
	splitter.SetMinimumPaneSize( 20 )
	self.DisableCmd( 'New' )
	self.DisableCmd( 'Cut' )
	self.DisableCmd( 'Copy' )
	self.DisableCmd( 'Paste' )
	self.DisableCmd( 'Close' )
	self.DisableCmd( 'NewFolder' )
	self.SetTitle( 'ZopeEDIT: *not connected*' )
	
    # --------------------------------------------------------------------
    def OnListItemSelected( self, event ):
	self._curitem = event.m_itemIndex
	
    # --------------------------------------------------------------------
    def OnListRightDown( self, event ):
	self.x = event.GetX()
	self.y = event.GetY()
#	event.Skip()
	
    # --------------------------------------------------------------------
    def OnListRightClick( self, event ):
	if self._curitem == -1:
	    return
	fname = self.flist.GetItemText( self._curitem )
	if not len(fname):
	    return
	menu = wxMenu()
	tpID1 = 0
	tpID2 = 1
	tpID3 = 2
	tpID4 = 3
	menu.Append( tpID1, "Edit %s" % fname )
	menu.Append( tpID2, "Download %s" % fname )
	menu.Append( tpID3, "Delete %s" % fname )
	menu.Append( tpID4, "Info" )
	EVT_MENU( self, tpID1, self.OnListDClick )
	EVT_MENU( self, tpID2, self.Download )
	EVT_MENU( self, tpID3, self.DeleteObject )
	EVT_MENU( self, tpID4, self.showInfo )
	self.flist.PopupMenu( menu, wxPoint( self.x, self.y ))
	menu.Destroy()

    def junk( self, event ):
	pass

    # --------------------------------------------------------------------
    def DeleteObject( self, event ):
	if self._curitem == -1:
	    return
	fname = self.flist.GetItemText( self._curitem )
	if not len(fname):
	    return
	dlg = wxMessageDialog( self, "Delete '%s'" % fname, 'Are You Sure', wxYES_NO | wxICON_QUESTION )
	if dlg.ShowModal() == wxID_YES:
	    dlg.Destroy()
	    try:
		self._ftp.delete( fname )
		self.flist.DeleteItem( self._curitem )
		self._curitem = -1
	    except ftplib.error_perm, msg:
		dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
		dlg.ShowModal()
		dlg.Destroy()
	else:
	    dlg.Destroy()
	    
    # --------------------------------------------------------------------
    def showInfo( self, event ):
	if self._curitem == -1:
	    return
	fname = self.flist.GetItemText( self._curitem )
	if not len(fname):
	    return
	if self.getFileInfo( fname ):
	    info = self._fileinfo
	    dlg = wxMessageDialog( self, 'Name: %s\nOwner: %s\nGroup: %s\nSize: %s\nPerms: %s' % (info[8], info[2], info[3], info[4], info[0]), 'File Info', wxOK | wxICON_INFORMATION )
	    dlg.ShowModal()
	    dlg.Destroy()
	
    
    # --------------------------------------------------------------------
    def oldOnListRightClick( self, event ):
	pass
	fname = self.flist.GetString( self._curitem )
	if sel == -1:
	    return
	if event.ShiftDown():
	    self.Download( fname )
	else:
	    if self.getFileInfo( fname ):
		info = self._fileinfo
		dlg = wxMessageDialog( self, 'Name: %s\nOwner: %s\nGroup: %s\nSize: %s\nPerms: %s' % (info[8], info[2], info[3], info[4], info[0]), 'File Info', wxOK | wxICON_INFORMATION )
		dlg.ShowModal()
		dlg.Destroy()
	    	    
    # --------------------------------------------------------------------
    def Download( self, event ):
	if self._curitem == -1:
	    return
	fname = self.flist.GetItemText( self._curitem )
	if not len(fname):
	    return
	fsize = 0
	if self.getFileInfo( fname ):
	    fsize = int(self._fileinfo[4])
	    self._progress = wxProgressDialog( 'Download: %s' % fname, '', fsize, self, wxPD_AUTO_HIDE | wxPD_APP_MODAL )
	    self._currentsize = 0
	try:
	    self._fp = open( DOWNLOAD_PATH + fname, 'w' )
	    self._ftp.retrbinary( 'RETR %s' % fname, self._saveFile_cb )
	    self._fp.close()
	    if self._progress != None:
		self._progress.Destroy()
		self._progress = None
	    dlg = wxMessageDialog( self, 'Download complete\n%s' % fname, 'Download', wxOK | wxICON_INFORMATION )
	    dlg.ShowModal()
	    dlg.Destroy()
	except ftplib.error_perm, msg:
	    if self._progress != None:
		self._progress.Destroy()
		self._progress = None
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()

    # --------------------------------------------------------------------
    def getFileInfo( self, fname ):
	self._fileinfo = []
	list = self._ftp.retrlines( 'LIST %s' % fname, self._getFileInfo_cb )
	if len( self._fileinfo ):
	    return 1
	return 0
	

    # --------------------------------------------------------------------
    def _getFileInfo_cb( self, line ):
	info = string.split( line )
	if len( info[0] ) == 10:
	    self._fileinfo = info
	
	
    # --------------------------------------------------------------------
    def OnListDClick( self, event ):
	if self._curitem == -1:
	    return
	fname = self.flist.GetItemText( self._curitem )
	if not len(fname):
	    return
	tmpfile = mktemp(TMP_FILE_SUFFIX)
	self._fp = open( tmpfile, 'w' )
	try:
	    self._ftp.retrbinary( 'RETR %s' % fname, self._saveFile_cb )
	    self._fp.close()
	    os.system( EDITOR_CMD % tmpfile )
	    dlg = wxMessageDialog( self, 'Save "%s" to Zope?' % ( fname ), 'Save', wxYES_NO | wxICON_QUESTION )
	    if dlg.ShowModal() == wxID_YES:
		try:
		    self._fp = open( tmpfile, 'r' )
		    self._ftp.storbinary( 'STOR %s' % fname, self._fp, 8192 )
		    self._fp.close()
		except ftplib.error_temp, msg:
		    dl1 = wxMessageDialog( self, msg + '\n[%s]' % fname, 'FTP Error', wxOK | wxICON_ERROR )
		    dl1.ShowModal()
		    dl1.Destroy()
	    dlg.Destroy()
	    os.unlink( tmpfile )
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()

    # --------------------------------------------------------------------
    def OnCut( self, event ):
	fname = self.flist.GetString(self.flist.GetSelection())
	tmpname = mktemp(TMP_FILE_SUFFIX)
	self._fp = open( tmpname, 'w' )
	try:
	    self._ftp.retrbinary( 'RETR %s' % fname, self._saveFile_cb )
	    self._fp.close()
	    self._ftp.delete( fname )
	    self._edit_info = []
	    self._edit_info.append( tmpname )
	    self._edit_info.append( fname )
	    self.EnableCmd( 'Paste' )
	    self.getFileList()
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()

    # --------------------------------------------------------------------
    def OnCopy( self, event ):
	fname = self.flist.getString(self.flist.GetSelection())
	tmpname = mktemp(TMP_FILE_SUFFIX)
	self._fp = open( tmpname, 'w' )
	try:
	    self._ftp.retrbinary( 'RETR %s' % fname, self._saveFile_cb )
	    self._fp.close()
	    self._edit_info = []
	    self._edit_info.append( tmpname )
	    self._edit_info.append( fname )
	    self.EnableCmd( 'Paste' )
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()
    
    # --------------------------------------------------------------------
    def OnPaste( self, event ):
	tmpname = self._edit_info[0]
	fname  = self._edit_info[1]
	try:
	    self._fp = open( tmpname, 'r' )
	    self._ftp.storbinary( 'STOR %s' % fname, self._fp, 8192 )
	    self._fp.close()
	    self.getFileList()
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()
    
    # --------------------------------------------------------------------
    def OnNew( self, event ):
	tmpfile = mktemp(TMP_FILE_SUFFIX)
	self._fp = open( tmpfile, 'w' )
	self._fp.write( '<dtml-var standard_html_header>\n' )
	self._fp.write( '<h2>FIX ME</h2>\n' )
	self._fp.write( '<dtml-var standard_html_footer>\n' )
	self._fp.close()
	try:
	    os.system( EDITOR_CMD % tmpfile )
	    dlg = wxTextEntryDialog( self, 'Document name', 'Save As', 'new_dtml' )
	    if dlg.ShowModal() == wxID_OK:
		fname = dlg.GetValue()
		try:
		    self._fp = open( tmpfile, 'r' )
		    self._ftp.storbinary( 'STOR %s' % fname, self._fp, 8192 )
		    self._fp.close()
		    self.getFileList()
		except ftplib.error_temp, msg:
		    dl1 = wxMessageDialog( self, msg + '\n[%s]' % fname, 'FTP Error', wxOK | wxICON_ERROR )
		    dl1.ShowModal()
		    dl1.Destroy()
	    dlg.Destroy()
	    os.unlink( tmpfile )
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()
	    
    # --------------------------------------------------------------------
    def OnNewFolder( self, event ):
	if self._rpc is None:
	    self.DisableCmd( 'NewFolder' )
	    return
	dlg = wxTextEntryDialog( self, 'New Folder', 'Folder Name', 'new_folder' )
	if dlg.ShowModal() == wxID_OK:
	    cmd = "self._rpc.%smanage_addFolder('%s','', 1, 1 )" % (self.pathToMethod(self._curpath),dlg.GetValue())
	    print cmd
	    try:
		eval( cmd )
		item = self.tree.AppendItem( self._curdir, dlg.GetValue() )
		self.tree.SetItemImage( item, self.img_folder )
	    except:
		print 'Failed'
	dlg.Destroy()
    
    # --------------------------------------------------------------------
    def _saveFile_cb( self, data ):
	self._fp.write( data )
	if self._progress != None:
	    self._currentsize = self._currentsize + len(data)
	    self._progress.Update( self._currentsize )
	    
    # --------------------------------------------------------------------
    def OnItemExpanded( self, event ):
	item = event.GetItem()
	
    # --------------------------------------------------------------------
    def OnItemCollapsed( self, event ):
	item = event.GetItem()

	
    # --------------------------------------------------------------------
    def OnAbout( self, event ):
	dlg = AboutBox( self )
	dlg.ShowModal()
	dlg.Destroy()
		    	
    # --------------------------------------------------------------------
    def OnHelp( self, event ):
	dlg = HelpBox( self )
	dlg.ShowModal()
	dlg.Destroy()
	
    # --------------------------------------------------------------------
    def OnTreeSelChanged( self, event ):
	# wxPython changed to GetItemParent in version 2.1.13
	if wxc.__version__ >= '2.1.13':
	    git = self.tree.GetItemParent
	else:
	    git = self.tree.GetParent
	item = event.GetItem()
	parent = git( item )
	path = '/' 
	if self.tree.GetItemText(item) != '/':
	    path = path + self.tree.GetItemText(item)
	while len(self.tree.GetItemText(parent)):
	    path = '/' + self.tree.GetItemText(parent) + path
	    parent = git( parent )
	self.tree.CollapseAndReset( item )
	self._curdir = item
	self.getFileList( path )
	
    # --------------------------------------------------------------------
    def OnOpen( self, event ):
	conn_str = '%s %s %s %s' % (HOST, PORT, USER, PASSWD)
	if USE_OPEN_DIALOG:
	    dlg = wxTextEntryDialog( self, 'Connect to: (host port user passwd)', 'Connect', conn_str )
	    if dlg.ShowModal() == wxID_OK:
		conn_str = dlg.GetValue()
	    else:
		return
	    dlg.Destroy()
	
	vals = string.split( conn_str )
	self._clearList()
	self.tree.DeleteAllItems()
	# connect to ftp
	try:
	    self._ftp.connect( vals[0], int(vals[1]))
	    self._ftp.login( vals[2], vals[3] )
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()
	    return
	# try RPC
	try:
	    self._rpc = xmlrpclib.Server("http://%s" % HOST, BasicAuthTransport( '%s' % USER, '%s' % PASSWD ))
	except:
	    self._rpc = None
	self.SetTitle( 'ZopeEDIT: %s:%s' % (vals[0], vals[1]) )
	self._root = self.tree.AddRoot( '/' )
	self._curdir = self._root
	self.getFileList('/')
	self.EnableCmd( 'New' )
	self.EnableCmd( 'Cut' )
	self.EnableCmd( 'Copy' )
	self.EnableCmd( 'Close' )
	self.EnableCmd( 'NewFolder' )
	
    # --------------------------------------------------------------------
    def OnClose( self, event ):
	self._clearList()
	self.tree.DeleteAllItems()
	self.SetTitle( 'ZopeEDIT: *not connected*' )
	self.DisableCmd( 'New' )
	self.DisableCmd( 'Cut' )
	self.DisableCmd( 'Copy' )
	self.DisableCmd( 'Close' )
	self.DisableCmd( 'NewFolder' )
	self._curpath = '/'
	self._ftp.close()

    # --------------------------------------------------------------------
    def _clearList( self ):
	self.flist.DeleteAllItems()
	self._curitem = -1
	
    # --------------------------------------------------------------------
    def getFileList( self, path=None ):
	self._clearList()
	try:
	    if path:
		self._ftp.cwd( path )
		self._curpath = path
	    list = self._ftp.retrlines( 'LIST', self.addFile )
	    self.tree.Expand( self._curdir )
	    self.flist.SetColumnWidth( 0, wxLIST_AUTOSIZE )
	    self.flist.SetColumnWidth( 1, wxLIST_AUTOSIZE )
	except ftplib.error_perm, msg:
	    dlg = wxMessageDialog( self, msg, 'FTP Error', wxOK | wxICON_ERROR )
	    dlg.ShowModal()
	    dlg.Destroy()

    # --------------------------------------------------------------------
    def addFile( self, line ):
	info = string.split( line )
	fname = info[len(info)-1]
	if len(fname) and fname != '..':
	    i = self.flist.GetItemCount()
	    if info[0][0] == 'd':
		self.flist.InsertImageStringItem( i, fname, self.img_folder )
	    else:
		self.flist.InsertImageStringItem( i, fname, self.img_text )
	    self.flist.SetStringItem( i, 1, self.getTitle(fname) )
	    if info[0][0] == 'd':
		item = self.tree.AppendItem( self._curdir, fname )
		self.tree.SetItemImage( item, self.img_folder )


    # --------------------------------------------------------------------
    def pathToMethod( self, path ):
	s = string.split( path, '/' )
	l = []
	for i in range( len(s) ):
	    if len(s[i]):
		l.append( s[i] )
	if len(l):
	    return string.join( l, '.' ) + '.'
	return ''
	    
    # --------------------------------------------------------------------
    def getTitle( self, fname ):
	if self._rpc is None:
	    return ''
	cmd = 'self._rpc.%s%s.title_or_id()' % (self.pathToMethod(self._curpath),fname)
	s = ''
	try:
	    s = eval( cmd )
	except:
	    return ''
	if s != fname:
	    return s
	return ''
	
	
# ========================================================================
class mApp( wxApp ):
    def OnInit( self ):
	frame = ZopeEDIT( ZopeEDIT_menu )
	frame.Show(true)
	self.SetTopWindow(frame)
	return true

# ========================================================================
def main():
    app = mApp(0)
    app.MainLoop()
    
# ========================================================================
# ------------------------------------------------------------------------
if __name__ == '__main__':
    main()
